"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const fs_1 = require("fs");
const path_1 = require("path");
class FileSchemaProvider {
    constructor(config) {
        this.config = config;
    }
    async resolveSchema() {
        if (this.schema)
            return this.schema;
        const { path } = this.config;
        let result;
        try {
            result = fs_1.readFileSync(path, {
                encoding: "utf-8"
            });
        }
        catch (err) {
            throw new Error(`Unable to read file ${path}. ${err.message}`);
        }
        const ext = path_1.extname(path);
        if (ext === ".json") {
            const parsed = JSON.parse(result);
            const __schema = parsed.data
                ? parsed.data.__schema
                : parsed.__schema
                    ? parsed.__schema
                    : parsed;
            this.schema = graphql_1.buildClientSchema({ __schema });
        }
        else if (ext === ".graphql" || ext === ".graphqls" || ext === ".gql") {
            const uri = `file://${path_1.resolve(path)}`;
            this.schema = graphql_1.buildSchema(new graphql_1.Source(result, uri));
        }
        if (!this.schema)
            throw new Error(`Schema could not be loaded for ${path}`);
        return this.schema;
    }
    onSchemaChange(_handler) {
        throw new Error("File watching not implemented yet");
        return () => { };
    }
}
exports.FileSchemaProvider = FileSchemaProvider;
//# sourceMappingURL=file.js.map