"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../../config");
const introspection_1 = require("./introspection");
const engine_1 = require("./engine");
const file_1 = require("./file");
function schemaProviderFromConfig(config, clientIdentity) {
    if (config_1.isServiceConfig(config)) {
        if (config.service.localSchemaFile) {
            return new file_1.FileSchemaProvider({ path: config.service.localSchemaFile });
        }
        if (config.service.endpoint) {
            return new introspection_1.IntrospectionSchemaProvider(config.service.endpoint);
        }
    }
    if (config_1.isClientConfig(config)) {
        if (typeof config.client.service === "string") {
            return new engine_1.EngineSchemaProvider(config, clientIdentity);
        }
        if (config.client.service) {
            if (config_1.isLocalServiceConfig(config.client.service)) {
                return new file_1.FileSchemaProvider({
                    path: config.client.service.localSchemaFile
                });
            }
            return new introspection_1.IntrospectionSchemaProvider(config.client.service);
        }
    }
    throw new Error("No schema provider was created, because the project type was unable to be resolved from your config. Please add either a client or service config. For more information, please refer to https://bit.ly/2ByILPj");
}
exports.schemaProviderFromConfig = schemaProviderFromConfig;
//# sourceMappingURL=index.js.map